//
//  XingAPIMessages.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+Messages.h"

@implementation XingAPI (Messages)

#pragma mark - Public methods

- (RACSignal *)addContact:(NSString *)contactID toConversation:(NSString *)conversationID {
    NSParameterAssert(contactID);
    NSParameterAssert(conversationID);
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@/participants/%@", self.session.xingUserID, conversationID, contactID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self putRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityConversation *conversation = [XingEntityConversation MR_findFirstByAttribute:@"conversationID" withValue:conversationID inContext:context];
            XingEntityUser *contact = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:contactID inContext:context];

            [conversation addParticipantsObject:contact];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)createConversation:(NSDictionary *)parameters {
    return [self createConversationForUser:nil parameters:parameters];
}

- (RACSignal *)createMessage:(NSDictionary *)parameters inConversation:(NSString *)conversationID {
    NSParameterAssert(conversationID);
    NSParameterAssert(parameters);
    NSParameterAssert(parameters[XingAPIParamContent]);
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@/messages", self.session.xingUserID, conversationID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self postRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityMessage *message = (XingEntityMessage *)response.resultObject;
            XingEntityConversation *conversation = [XingEntityConversation MR_findFirstByAttribute:@"conversationID" withValue:conversationID inContext:context];
            [conversation addMessagesObject:message];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)deleteConversation:(NSString *)conversationID {
    NSParameterAssert(conversationID);
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@", self.session.xingUserID, conversationID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self deleteRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityConversation *conversation = [XingEntityConversation MR_findFirstByAttribute:@"conversationID" withValue:conversationID inContext:context];
            [context deleteObject:conversation];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadConversation:(NSString *)conversationID {
    return [self loadConversation:conversationID forUser:nil parameters:nil];
}

- (RACSignal *)loadConversation:(NSString *)conversationID parameters:(NSDictionary *)parameters {
    return [self loadConversation:conversationID forUser:nil parameters:parameters];
}

- (RACSignal *)loadConversations {
    return [self loadConversationsForUser:nil parameters:nil];
}

- (RACSignal *)loadConversations:(NSDictionary *)parameters {
    return [self loadConversationsForUser:nil parameters:parameters];
}

- (RACSignal *)loadMessage:(NSString *)messageID conversation:(NSString *)conversationID {
    return [self loadMessage:messageID conversation:conversationID user:nil parameters:nil];
}

- (RACSignal *)loadMessage:(NSString *)messageID conversation:(NSString *)conversationID parameters:(NSDictionary *)parameters {
    return [self loadMessage:messageID conversation:conversationID user:nil parameters:parameters];
}

- (RACSignal *)loadMessages:(NSString *)conversationID {
    return [self loadMessagesForConversation:conversationID forUser:nil parameters:nil];
}

- (RACSignal *)loadMessages:(NSString *)conversationID parameters:(NSDictionary *)parameters {
    return [self loadMessagesForConversation:conversationID forUser:nil parameters:parameters];
}

- (RACSignal *)markAllMessagesAsRead:(NSString *)conversationID {
    NSParameterAssert(conversationID);
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@/read", self.session.xingUserID, conversationID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self putRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityConversation *conversation = [XingEntityConversation MR_findFirstByAttribute:@"conversationID" withValue:conversationID inContext:context];
            for (XingEntityMessage *message in conversation.messages) {
                message.read = @YES;
            }

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)markMessageAsRead:(NSString *)messageID inConversation:(NSString *)conversationID {
    NSParameterAssert(messageID);
    NSParameterAssert(conversationID);
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@/messages/%@/read", self.session.xingUserID, conversationID, messageID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self putRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityMessage *message = [XingEntityMessage MR_findFirstByAttribute:@"messageID" withValue:messageID inContext:context];
            message.read = @YES;

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)markMessageAsUnread:(NSString *)messageID inConversation:(NSString *)conversationID {
    NSParameterAssert(messageID);
    NSParameterAssert(conversationID);
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@/messages/%@/read", self.session.xingUserID, conversationID, messageID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self deleteRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityMessage *message = [XingEntityMessage MR_findFirstByAttribute:@"messageID" withValue:messageID inContext:context];
            message.read = @NO;

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

#pragma mark - Private methods

- (RACSignal *)createConversationForUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    NSParameterAssert(parameters);
    NSParameterAssert(parameters[XingAPIParamContent]);
    NSParameterAssert(parameters[XingAPIParamRecipientIDS]);
    NSParameterAssert(parameters[XingAPIParamSubject]);
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self postRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingEntityConversation *conversation = (XingEntityConversation *)response.resultObject;
            [user addConversationsObject:conversation];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadConversation:(NSString *)conversationID
                        forUser:(NSString *)userID
                     parameters:(NSDictionary *)parameters {
    NSParameterAssert(conversationID);
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@", userID, conversationID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingEntityConversation *conversation = (XingEntityConversation *)response.resultObject;
            [user addConversationsObject:conversation];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadConversationsForUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingObjectConversations *conversations = (XingObjectConversations *)response.resultObject;
            [user addConversations:conversations.items];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadMessagesForConversation:(NSString *)conversationID
                                   forUser:(NSString *)userID
                                parameters:(NSDictionary *)parameters {
    NSParameterAssert(conversationID);
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@/messages", userID, conversationID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingEntityConversation *conversation = [XingEntityConversation MR_findFirstByAttribute:@"conversationID" withValue:conversationID inContext:context];
            XingObjectMessages *messages = (XingObjectMessages *)response.resultObject;

            [conversation addMessages:messages.items];
            [user addConversationsObject:conversation];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadMessage:(NSString *)messageID
              conversation:(NSString *)conversationID
                      user:(NSString *)userID
                parameters:(NSDictionary *)parameters {
    NSParameterAssert(conversationID);
    NSParameterAssert(messageID);
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/conversations/%@/messages/%@", userID, conversationID, messageID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingEntityConversation *conversation = [XingEntityConversation MR_findFirstByAttribute:@"conversationID" withValue:conversationID inContext:context];
            XingEntityMessage *message = (XingEntityMessage *)response.resultObject;

            [conversation addMessagesObject:message];
            [user addConversationsObject:conversation];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)validateRecipient:(NSString *)userID {
    return nil;
}

@end
