//
//  XingAPINetworkFeed.h
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI.h"

@interface XingAPI (NetworkFeed)

/**
 https://dev.xing.com/docs/post/activities/:activity_id/comments

 Creates a comment for a certain activity. Only activities with COMMENT in the possible_actions
 field can be commented on.

 @param activityID The activityID (id) of the activity the comment should be created for.
 @param parameters A dictionary of parameters. See the documentation linked above for valid
 parameters and their meaning.
 */
- (RACSignal *)createComment:(NSDictionary *)parameters inActivity:(NSString *)activityID;

/**
 https://dev.xing.com/docs/put/activities/:activity_id/like

 Adds the current user to the list of likes for the given activity. It's only possible to like
 activities which have `LIKE` in the `possible_actions` field.

 @param activityID The activityID (activity_id) of the activity the like should be created for.
 */
- (RACSignal *)createLikeOfActivity:(NSString *)activityID;

/**
 https://dev.xing.com/docs/delete/activities/:id

 Deletes the activity with the given ID. Users can only delete their own activities, and only
 activities with `DELETE` in the `possible_actions` field can be deleted.

 @param activityID The activityID (id) of the activity that should be deleted.
 */
- (RACSignal *)deleteActivity:(NSString *)activityID;

/**
 https://dev.xing.com/docs/delete/activities/:activity_id/comments/:id

 Deletes a comment for a certain activity. Users can only delete their own comments or comments for
 activities they own.

 @param activityID The activityID (activity_id) of the activity the comment belongs to.
 @param commentID The commentID (id) of the comment that should be deleted.
 */
- (RACSignal *)deleteComment:(NSString *)commentID inActivity:(NSString *)activityID;

/**
 https://dev.xing.com/docs/delete/activities/:activity_id/like

 Removes a like the current user already added to the given activity. It's only possible to like
 activities which have `LIKE` in the `possible_actions` field.

 @param activityID The activityID (activity_id) of the activity the like should be deleted from.
 */
- (RACSignal *)deleteLikeOfActivity:(NSString *)activityID;

/**
 https://dev.xing.com/docs/get/activities/:id

 Returns a single activity. The response format is the same as the one in the network or user feed,
 even though a single activity will never be aggregated.

 @param activityID The activityID (id) of the activity that should be fetched.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingEntityActivity in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadActivity:(NSString *)activityID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadActivity:(NSString *)activityID;

/**
 https://dev.xing.com/docs/get/activities/:activity_id/comments

 Returns a list of comments added to the activity with the given activity_id. This list is sorted
 by the creation date of the comments.

 @param activityID The activityID (id) of the activity the comments should be fetched for.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingEntityActivityComment in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadActivityComments:(NSString *)activityID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadActivityComments:(NSString *)activityID;

/**
 https://dev.xing.com/docs/get/activities/:activity_id/likes

 Returns a list of users who liked a certain activity.

 @param activityID The activityID (id) of the activity the likes should be fetched for.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingEntityUser in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadActivityLikes:(NSString *)activityID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadActivityLikes:(NSString *)activityID;

/**
 https://dev.xing.com/docs/get/users/:user_id/network_feed

 Returns the stream of activities recently performed by the user's network.

 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingEntityActivity in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadNetworkFeed:(NSDictionary *)parameters;
- (RACSignal *)loadNetworkFeed;

/**
 https://dev.xing.com/docs/get/users/:id/feed

 Returns the stream of activities recently performed by the corresponding user.

 @param userID The userID (id) of user whose feed is to be returned.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingEntityActivity in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadUserFeedForUser:(NSString *)userID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadUserFeedWithParameters:(NSDictionary *)parameters;
- (RACSignal *)loadUserFeedForUser:(NSString *)userID;
- (RACSignal *)loadUserFeed;

/**
 https://dev.xing.com/docs/post/activities/:id/share

 Sharing an activity means recommending it to your network. This will then create a new activity for
 the current user. Only activities with `SHARE` in the `possible_actions` field can be shared.

 @param activityID The activityID (id) of the activity that should be shared.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 */
- (RACSignal *)shareActivity:(NSString *)activityID parameters:(NSDictionary *)parameters;
- (RACSignal *)shareActivity:(NSString *)activityID;

@end
