//
//  XingAPI.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/1/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingAPI.h"

@implementation XingAPI

- (id)initWithManager:(XingObjectManager *)inManager session:(XingSession *)inSession {
    if (self = [super init]) {
        _globalSignal = [RACSubject subject];
        _manager = inManager;
        _session = inSession;
    }
    return self;
}

# pragma mark - Private methods

- (NSManagedObjectContext *)context {
    if (_context == nil) {
        _context = self.manager.managedObjectStore.persistentStoreManagedObjectContext;
    }
    return _context;
}

- (RACSignal *)enqueueObjectRequestOperation:(RKManagedObjectRequestOperation *)operation {
    __weak __typeof(&*self)weakSelf = self;
    RACReplaySubject *subject = [RACReplaySubject subject];

    operation.managedObjectContext = self.context;

    [operation setCompletionBlockWithSuccess:^(RKObjectRequestOperation *inOperation, RKMappingResult *mappingResult) {
        RKManagedObjectRequestOperation *operation = (RKManagedObjectRequestOperation *)inOperation;
        [subject sendNext:[XingAPIResponse responseWithOperation:operation result:mappingResult]];
        [subject sendCompleted];
    } failure:^(RKObjectRequestOperation *operation, NSError *error) {
        [subject sendError:[weakSelf transformError:error operation:operation]];
    }];

    [operation setWillMapDeserializedResponseBlock:^id(id deserializedResponseBody) {
        return [weakSelf transformResponse:deserializedResponseBody];
    }];

    [subject subscribeNext:^(id x) {
        [weakSelf.globalSignal sendNext:x];
    } error:^(NSError *error) {
        [weakSelf.globalSignal sendError:error];
    }];

    [self.manager enqueueObjectRequestOperation:operation];

    return subject;
}

- (RACSignal *)deleteRequestAtPath:(NSString *)path parameters:(NSDictionary *)parameters {
    RKManagedObjectRequestOperation *operation = [self.manager appropriateObjectRequestOperationWithObject:nil method:RKRequestMethodDELETE path:path parameters:parameters];
    return [self enqueueObjectRequestOperation:operation];
}

- (RACSignal *)getRequestAtPath:(NSString *)path parameters:(NSDictionary *)parameters {
    RKManagedObjectRequestOperation *operation = [self.manager appropriateObjectRequestOperationWithObject:nil method:RKRequestMethodGET path:path parameters:parameters];
    return [self enqueueObjectRequestOperation:operation];
}

- (RACSignal *)postRequestAtPath:(NSString *)path parameters:(NSDictionary *)parameters {
    RKManagedObjectRequestOperation *operation = [self.manager appropriateObjectRequestOperationWithObject:nil method:RKRequestMethodPOST path:path parameters:parameters];
    return [self enqueueObjectRequestOperation:operation];
}

- (RACSignal *)putRequestAtPath:(NSString *)path parameters:(NSDictionary *)parameters {
    RKManagedObjectRequestOperation *operation = [self.manager appropriateObjectRequestOperationWithObject:nil method:RKRequestMethodPUT path:path parameters:parameters];
    return [self enqueueObjectRequestOperation:operation];
}

- (NSError *)transformError:(NSError *)error operation:(RKObjectRequestOperation *)operation {
    XingErrorMessage *xingError = (XingErrorMessage *)error.userInfo[RKObjectMapperErrorObjectsKey][0];
    if (xingError != nil) {
        NSDictionary *userInfo = @{NSLocalizedDescriptionKey: xingError.errorMessage, XingAPIErrorKey: xingError};
        return [[NSError alloc] initWithDomain:@"com.xing.XingSDK.HTTPErrorDomain"
                                          code:operation.HTTPRequestOperation.response.statusCode
                                      userInfo:userInfo];
    }
    return error;
}

- (id)transformResponse:(id)response {
    if ([response isKindOfClass:NSDictionary.class] && (response[@"network_activities"] || response[@"activities"])) {
        return [self transformResponseNetworkActivities:response];
    }
    return response;
}

- (id)transformResponseNetworkActivities:(id)response {
    NSMutableDictionary *data = [(NSDictionary *)response mutableDeepCopy];
    if ([data[@"network_activities"] isKindOfClass:NSArray.class]) {
        for (NSMutableDictionary *row in data[@"network_activities"]) {
            row[@"ids"] = [(NSArray *)row[@"ids"] componentsJoinedByString:@","];
        }
    }
    if ([data[@"activities"] isKindOfClass:NSArray.class]) {
        for (NSMutableDictionary *row in data[@"activities"]) {
            row[@"ids"] = [(NSArray *)row[@"ids"] componentsJoinedByString:@","];
        }
    }
    return data;
}

- (void)withLimit:(int)limit andTotal:(int)total callBlock:(void (^)(int offset))block {
    int offset = 0;
    do {
        block(offset);
        offset += limit;
    } while (offset < total);
}

@end
