//
//  XingSDK.h
//  XingSDK
//
//  Created by Johannes Plunien on 11/13/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import <CoreData/CoreData.h>
#import <Foundation/Foundation.h>
#import <MagicalRecord/CoreData+MagicalRecord.h>
#import <ReactiveCocoa/ReactiveCocoa.h>
#import <RestKit/RestKit.h>

#import "XingAPI.h"
#import "XingAPICategories.h"
#import "XingAPIResponse.h"
#import "XingEntities.h"
#import "XingHTTPClient.h"
#import "XingObjectManager.h"
#import "XingSession.h"

/**
 The `XingSDK` class provides an interface for accessing the public REST API of
 [xing.com](https://www.xing.com/), see also: https://dev.xing.com/

 There is always a shared instance of `XingSDK` with a set of class methods, to get access to the
 other objects. The shared instance uses by default the production base URL of the REST API:
 https://api.xing.com/v1

 If the `XingSDK` should use a different URL, the shared instance must be replaced:

    XingSDK.sharedInstance = [[XingSDK alloc] initWithURL:@"https://internal.xing.com/api/v1"];

 ## Class structure

 * `XingSDK.api` - XingAPI - Provides syntactic sugar around [RestKit](http://restkit.org/) to
 access the API
 * `XingSDK.session` - XingSession - Handles the authentication via oAuth

 The XingAPI is suposed to provide access to all API calls, with all flexiblity that is necessary.
 It is just a thin wrapper around the great RestKit framework. However, if there is not enough
 flexibility or some missing implementation for an API call, the XingObjectManager can be used
 directly via the `XingSDK.manager` getter. If the object mapping (and the storage in core data)
 should be bypassed `XingSDK.client` can be used directly, which is just an AFHTTPClient that uses
 XingSession for (oAuth) signing the HTTP requests.

 ## Synopsis

 Here are three examples, from the highest level (XingAPI sugar), via the RestKit object manager
 down to using the XingHTTPClient directly. The preferred way should always be the XingAPI one.

 ### Setup oAuth

    XingSDK.session.consumerKey = @"foo";
    XingSDK.session.consumerSecret = @"bar";
    XingSDK.session.accessToken = @"bla";
    XingSDK.session.accessTokenSecret = @"fasel";
    XingSDK.session.xingUserID = @"007_f00b4r";

 #### Using XingAPI

    [XingSDK.api.contacts.loadContacts subscribeNext:^(XingAPIResponse *response) {
        XingObjectContacts *contacts = response.resultObject;
        NSLog(@"Fetched %u/%@ contacts", contacts.items.count, contacts.total);
        for (XingEntityUser *contact in contacts.items) {
            NSLog(@"Contact: %@", contact);
        }
    } error:^(NSError *error) {
        NSLog(@"error: %@", error);
    }];

 #### Using XingObjectManager

    [XingSDK.manager getObjectsAtPath:@"users/me/contacts" parameters:nil success:^(RKObjectRequestOperation *operation, RKMappingResult *mappingResult) {
        NSLog(@"Mapped objects: %@", mappingResult);
    } failure:^(RKObjectRequestOperation *operation, NSError *error) {
        NSLog(@"error: %@", error);
    }];

 #### Using XingHTTPClient

    [XingSDK.client getPath:@"users/me/contacts" parameters:nil success:^(AFHTTPRequestOperation *operation, id result) {
        NSLog(@"Deserialized JSON result: %@", result);
    } failure:^(AFHTTPRequestOperation *operation, NSError *error) {
        NSLog(@"error: %@", error);
    }];

 */
@interface XingSDK : NSObject

@property (nonatomic, strong, readonly) NSString *baseURL;
@property (nonatomic, strong, readonly) XingObjectManager *manager;
@property (nonatomic, strong, readonly) XingAPI *api;
@property (nonatomic, strong, readonly) XingSession *session;

///----------------------------------------
/// @name Initialize with a custom base URL
///----------------------------------------

/**
 Return an instance of XingSDK with a custom base URL.

 @param inBaseURL The base URL that should be used.
 @return A new XingSDK instance.
 */
- (id)initWithURL:(NSString *)inBaseURL;

///--------------------------------------------
/// @name Access objects of the shared instance
///--------------------------------------------

/**
 Return the XingAPI object that is used by the shared instance.

 @return The XingAPI object that is used by the shared instance.
 */
+ (XingAPI *)api;

/**
 Return the XingHTTPClient object that is used by the shared instance.

 @return The XingHTTPClient object that is used by the shared instance.
 */
+ (XingHTTPClient *)client;

/**
 Return the XingObjectManager object that is used by the shared instance. See also:
 [RKObjectManager](http://restkit.org/api/0.20.0/Classes/RKObjectManager.html)

 @return The XingObjectManager object that is used by the shared instance.
 */
+ (XingObjectManager *)manager;

/**
 Return the XingSession object that is used by the shared instance.

 @return The XingSession object that is used by the shared instance.
 */
+ (XingSession *)session;

///------------------------------------
/// @name Replacing the shared instance
///------------------------------------

/**
 Set/replace the XingSDK shared instance.

 @param inSharedInstance The new shared XingSDK instance.
 */
+ (void)setSharedInstance:(XingSDK *)inSharedInstance;

@end
