//
//  XingSDK.m
//  XingSDK
//
//  Created by Johannes Plunien on 11/13/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingSDK.h"

@interface NSManagedObjectContext (XingSDK)

+ (void) MR_setDefaultContext:(NSManagedObjectContext *)moc;
+ (void) MR_setRootSavingContext:(NSManagedObjectContext *)context;

@end

@interface XingSDK ()

@end

@implementation XingSDK

static XingSDK *_sharedInstance = nil;

- (id)init {
	return [self initWithURL:@"https://api.xing.com/v1"];
}

- (id)initWithURL:(NSString *)inBaseURL {
    NSParameterAssert(inBaseURL);
    if (self = [super init]) {
        XingHTTPClient *client;
        _baseURL = inBaseURL;
        _session = [[XingSession alloc] initWithBaseURL:inBaseURL];
        client = [[XingHTTPClient alloc] initWithBaseURL:[NSURL URLWithString:inBaseURL] session:_session];
        [client registerHTTPOperationClass:[AFJSONRequestOperation class]];
        [client setDefaultHeader:@"Accept" value:RKMIMETypeJSON];
        _manager = [[XingObjectManager alloc] initWithHTTPClient:client];
        _api = [[XingAPI alloc] initWithManager:_manager session:_session];
        [self setupRestKitStore];
        [self setupRestKitMapping];

        if (_sharedInstance == nil) {
            [[self class] setSharedInstance:self];
        }
}
    return self;
}

#pragma mark - api getter

+ (XingAPI *)api {
    return (XingAPI *)[[self sharedInstance] api];
}

#pragma mark - client getter

+ (XingHTTPClient *)client {
    return (XingHTTPClient *)[[[self sharedInstance] manager] HTTPClient];
}

#pragma mark - manager getter

+ (XingObjectManager *)manager {
    return [[self sharedInstance] manager];
}

#pragma mark - session getter

+ (XingSession *)session {
    return [[self sharedInstance] session];
}

#pragma mark - sharedInstance getter/setter

+ (id)sharedInstance {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        if (_sharedInstance == nil) {
            _sharedInstance = [[self alloc] init];
        }
    });
    return _sharedInstance;
}

+ (void)setSharedInstance:(XingSDK *)inSharedInstance {
    NSParameterAssert(inSharedInstance);
    _sharedInstance = inSharedInstance;
}

#pragma mark - private methods

- (NSString *)description {
    return [NSString stringWithFormat:@"<%@:%p baseURL:%@>", NSStringFromClass([self class]), self, self.baseURL];
}

#pragma mark - RestKit mapping

- (void)setupRestKitMapping {
    [RKEntityMapping setEntityIdentificationInferenceEnabled:NO];
    [self.manager addResponseDescriptor:[self restKitErrorDescriptor]];
    [self.manager addResponseDescriptorsFromArray:[self restKitUserProfilesDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitJobsDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitMessagesDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitStatusMessagesDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitProfileMessagesDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitContactsDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitContactRequestsDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitContactPathDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitBookmarksDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitNetworkFeedDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitProfileVisitsDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitInvitationsDescriptors]];
    [self.manager addResponseDescriptorsFromArray:[self restKitGeoLocationsDescriptors]];
}

#pragma mark - RestKit response descriptors

- (RKResponseDescriptor *)restKitErrorDescriptor {
    // Error mapping
    return [RKResponseDescriptor responseDescriptorWithMapping:[XingErrorMessage mapping]
                                                   pathPattern:nil
                                                       keyPath:nil
                                                   statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassClientError)];
}

- (NSArray *)restKitUserProfilesDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:id
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityUser mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID"
                                                keyPath:@"users"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/me/id_card
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityUser mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/me/id_card"
                                                keyPath:@"id_card"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/find_by_emails
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectFindByEmails mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/find_by_emails"
                                                keyPath:@"results"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/:user_id/network/recommendations
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectUserRecommendations mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/network/recommendations"
                                                keyPath:@"user_recommendations"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitJobsDescriptors {
    return @[
    // https://dev.xing.com/docs/get/jobs/:id
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityJob mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"jobs/:jobID"
                                                keyPath:@"job"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/jobs/find
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectJobs mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"jobs/find"
                                                keyPath:@"jobs"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/:user_id/jobs/recommendations
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectJobRecommendations mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/jobs/recommendations"
                                                keyPath:@"job_recommendations"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitMessagesDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:user_id/conversations
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectConversations mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/conversations"
                                                keyPath:@"conversations"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/:user_id/conversations/:id
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityConversation mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/conversations/:conversationID"
                                                keyPath:@"conversation"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/:user_id/conversations/:id/messages
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectMessages mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/conversations/:conversationID/messages"
                                                keyPath:@"messages"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/:user_id/conversations/:conversation_id/messages/:id
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityMessage mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/conversations/:conversationID/messages/:messageID"
                                                keyPath:@"message"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/post/users/:user_id/conversations
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityConversation mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/conversations"
                                                keyPath:@"conversation"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/post/users/:user_id/conversations/:conversation_id/messages
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityMessage mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/conversations/:conversationID/messages"
                                                keyPath:@"message"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitStatusMessagesDescriptors {
    return @[];
}

- (NSArray *)restKitProfileMessagesDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:user_id/profile_message
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityProfileMessage mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/profile_message"
                                                keyPath:@"profile_message"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitContactsDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:user_id/contacts
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectContacts mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/contacts"
                                                keyPath:@"contacts"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/:user_id/contacts/shared
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectContacts mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/contacts/shared"
                                                keyPath:@"shared_contacts"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitContactRequestsDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:user_id/contact_requests
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityContactRequest mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/contact_requests"
                                                keyPath:@"contact_requests"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitContactPathDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:user_id/network/:other_user_id/paths
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectContactPaths mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/network/:otherUserID/paths"
                                                keyPath:@"contact_paths"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitBookmarksDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:user_id/bookmarks
    [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectBookmarks mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/bookmarks"
                                                keyPath:@"bookmarks"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitNetworkFeedDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:user_id/network_feed
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityActivity mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/network_feed"
                                                keyPath:@"network_activities"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/users/:id/feed
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityActivity mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/feed"
                                                keyPath:@"network_activities"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/activities/:id
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityActivity mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"activities/:activityID"
                                                keyPath:@"activities"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/activities/:activity_id/comments
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityActivityComment mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"activities/:activityID/comments"
                                                keyPath:@"comments"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],

    // https://dev.xing.com/docs/get/activities/:activity_id/likes
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityUser mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"activities/:activityID/likes"
                                                keyPath:@"likes"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitProfileVisitsDescriptors {
    return @[
    // https://dev.xing.com/docs/get/users/:user_id/visits
    [RKResponseDescriptor responseDescriptorWithMapping:[XingEntityProfileVisit mappingInManagedObjectStore:self.manager.managedObjectStore]
                                            pathPattern:@"users/:userID/visits"
                                                keyPath:@"visits"
                                            statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitInvitationsDescriptors {
    return @[
     // https://dev.xing.com/docs/post/users/invite
     [RKResponseDescriptor responseDescriptorWithMapping:[XingObjectInvitationStats mappingInManagedObjectStore:self.manager.managedObjectStore]
                                             pathPattern:@"users/invite"
                                                 keyPath:@"invitation_stats"
                                             statusCodes:RKStatusCodeIndexSetForClass(RKStatusCodeClassSuccessful)],
    ];
}

- (NSArray *)restKitGeoLocationsDescriptors {
    return @[];
}

#pragma mark - RestKit store

- (void)setupRestKitStore {
    NSMutableArray *allBundles = [NSMutableArray array];
    [allBundles addObjectsFromArray:[NSBundle allBundles]];
    [allBundles addObjectsFromArray:[NSBundle allFrameworks]];

    NSManagedObjectModel *managedObjectModel = [NSManagedObjectModel mergedModelFromBundles:allBundles];
    RKManagedObjectStore *managedObjectStore = [[RKManagedObjectStore alloc] initWithManagedObjectModel:managedObjectModel];
    self.manager.managedObjectStore = managedObjectStore;

    [managedObjectStore createPersistentStoreCoordinator];
    NSString *storePath = [RKApplicationDataDirectory() stringByAppendingPathComponent:@"XingSDK.sqlite"];
    NSError *error;
    NSPersistentStore *persistentStore = [managedObjectStore addSQLitePersistentStoreAtPath:storePath
                                                                     fromSeedDatabaseAtPath:nil
                                                                          withConfiguration:nil
                                                                                    options:nil
                                                                                      error:&error];
    NSAssert(persistentStore, @"Failed to add persistent store (%@) with error: %@", storePath, error);

    // Create the managed object contexts
    [managedObjectStore createManagedObjectContexts];

    // Configure a managed object cache to ensure we do not create duplicate objects
    managedObjectStore.managedObjectCache = [[RKInMemoryManagedObjectCache alloc] initWithManagedObjectContext:managedObjectStore.persistentStoreManagedObjectContext];

    [NSManagedObjectContext MR_setRootSavingContext:managedObjectStore.persistentStoreManagedObjectContext];
    [NSManagedObjectContext MR_setDefaultContext:managedObjectStore.mainQueueManagedObjectContext];
}

@end
