//
//  XingSession.h
//  XingSDK
//
//  Created by Johannes Plunien on 11/18/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <RestKit/RestKit.h>
#import "GTMOAuthViewControllerTouch.h"

/**
 The XING API requires authentication via oAuth. You can use any library for the authentication, but
 you then must provide all details to `XingSDK.session`:

    XingSDK.session.consumerKey = @"foo";
    XingSDK.session.consumerSecret = @"bar";
    XingSDK.session.accessToken = @"bla";
    XingSDK.session.accessTokenSecret = @"fasel";
    XingSDK.session.xingUserID = @"007_f00b4r";

 The 2nd option is to just provide the `consumerKey` and `consumerSecret` and let us handle the
 rest for you. We have `GTMOAuthViewControllerTouch` included for handling the authentication
 quickly:

    XingSDK.session.consumerKey = @"foo";
    XingSDK.session.consumerSecret = @"bar";

    if (XingSDK.session.isReady == NO) {
        UIViewController *avc = [XingSDK.session authViewController:^(BOOL authorized, NSError *error) {
            if (authorized) {
                // Show your logged in view controller
            }
        }];
        // Assuming we are in a view controller here
        [self.navigationController pushViewController:avc animated:YES];
    }

 No matter if you use `GTMOAuthViewControllerTouch` or your own authentication stuff, we will
 always use `GTMOauthAuthentication` for signing the requests and storing the oAuth tokens securely
 in the keychain.
*/
@interface XingSession : NSObject

typedef void (^AuthCompletionBlock)(BOOL authorized, NSError *error);

@property (nonatomic, strong, readonly) NSString *baseURL;

- (GTMOAuthViewControllerTouch *)authViewController:(AuthCompletionBlock)completionBlock;
- (id)initWithBaseURL:(NSString *)inBaseURL;
- (BOOL)isReady;
- (void)logout;

#pragma mark - oAuth secret/key setters

- (NSString *)consumerKey;
- (NSString *)consumerSecret;

- (void)setConsumerKey:(NSString *)inConsumerKey;
- (void)setConsumerSecret:(NSString *)inConsumerSecret;

- (NSString *)accessToken;
- (NSString *)accessTokenSecret;

- (void)setAccessToken:(NSString *)inAccessToken;
- (void)setAccessTokenSecret:(NSString *)inAccessTokenSecret;

- (NSString *)callback;
- (void)setCallback:(NSString *)inCallback;

- (NSString *)xingUserID;
- (void)setXingUserID:(NSString *)inXingUserID;

#pragma mark - oAuth signing

- (BOOL)signRequest:(NSMutableURLRequest *)request;

@end
