//
//  XingSession.m
//  XingSDK
//
//  Created by Johannes Plunien on 11/18/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import <RestKit/RestKit.h>
#import "XingSession.h"
#import "XingOAuthAuthentication.h"

#define kAppName [[NSBundle mainBundle] bundleIdentifier]

@interface XingSession ()

@property (nonatomic, strong) XingOAuthAuthentication *auth;
@property (nonatomic, copy) AuthCompletionBlock block;

@end

@implementation XingSession

- (id)initWithBaseURL:(NSString *)inBaseURL {
    if (self = [super init]) {
        _baseURL = inBaseURL;
        _auth = [[XingOAuthAuthentication alloc] initWithSignatureMethod:kGTMOAuthSignatureMethodHMAC_SHA1
                                                             consumerKey:self.auth.consumerKey
                                                              privateKey:self.auth.privateKey];
        _auth.serviceProvider = kAppName;
        _auth.callback = [NSString stringWithFormat:@"%@://callback", kAppName];
        [GTMOAuthViewControllerTouch authorizeFromKeychainForName:kAppName
                                                   authentication:_auth];
    }
    return self;
}

- (GTMOAuthViewControllerTouch *)authViewController:(AuthCompletionBlock)inBlock {
    NSAssert(self.consumerKey != nil, @"The consumerKey must be set on the XingSession object");
    NSAssert(self.consumerSecret != nil, @"The consumerSecret must be set on the XingSession object");
    self.block = inBlock;
    return [self authViewController];
}

- (BOOL)isReady {
    return (
            self.consumerKey != nil &&
            self.consumerSecret != nil &&
            self.accessToken != nil &&
            self.accessTokenSecret != nil &&
            self.xingUserID != nil &&
            [self.auth canAuthorize]
            );
}

- (void)logout {
    [GTMOAuthViewControllerTouch removeParamsFromKeychainForName:kAppName];
    [self clearCookies];
}

#pragma mark - oAuth secret/key setters

- (NSString *)consumerKey {
    return self.auth.consumerKey;
}

- (NSString *)consumerSecret {
    return self.auth.privateKey;
}

- (void)setConsumerKey:(NSString *)inConsumerKey {
    self.auth.consumerKey = inConsumerKey;
}

- (void)setConsumerSecret:(NSString *)inConsumerSecret {
    self.auth.privateKey = inConsumerSecret;
}

- (NSString *)accessToken {
    return self.auth.accessToken;
}

- (NSString *)accessTokenSecret {
    return self.auth.tokenSecret;
}

- (void)setAccessToken:(NSString *)inAccessToken {
    self.auth.accessToken = inAccessToken;
}

- (void)setAccessTokenSecret:(NSString *)inAccessTokenSecret {
    self.auth.tokenSecret = inAccessTokenSecret;
}

- (NSString *)callback {
    return self.auth.callback;
}

- (void)setCallback:(NSString *)inCallback {
    self.auth.callback = inCallback;
}

- (NSString *)xingUserID {
    return self.auth.xingUserID;
}

- (void)setXingUserID:(NSString *)inXingUserID {
    self.auth.xingUserID = inXingUserID;
}

#pragma mark - oAuth signing

- (BOOL)signRequest:(NSMutableURLRequest *)request {
    return [self.auth authorizeRequest:request];
}

#pragma mark - Private methods

- (NSURL *)accessTokenURL {
    return [NSURL URLWithString:[NSString stringWithFormat:@"%@/access_token", self.baseURL]];
}

- (NSURL *)authorizeURL {
    return [NSURL URLWithString:[NSString stringWithFormat:@"%@/authorize", self.baseURL]];
}

- (GTMOAuthViewControllerTouch *)authViewController {
    return [[GTMOAuthViewControllerTouch alloc] initWithScope:nil
                                                     language:nil
                                              requestTokenURL:[self requestTokenURL]
                                            authorizeTokenURL:[self authorizeURL]
                                               accessTokenURL:[self accessTokenURL]
                                               authentication:self.auth
                                               appServiceName:kAppName
                                                     delegate:self
                                             finishedSelector:@selector(viewController:finishedWithAuth:error:)];
}

- (void)clearCookies {
    NSString *baseUrl = self.baseURL;
    NSString *path = [[NSURL URLWithString:baseUrl] path];
    baseUrl = [baseUrl stringByReplacingOccurrencesOfString:path withString:@""];

    NSArray *clearCookieUrls = @[
    @"https://api.xing.com",
    @"https://login.xing.com",
    baseUrl,
    ];

    NSHTTPCookieStorage *cookieStorage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
    for (NSString *url in clearCookieUrls) {
        NSArray *cookies =  [cookieStorage cookiesForURL:[NSURL URLWithString:url]];
        for (NSHTTPCookie *cookie in cookies) {
            [cookieStorage deleteCookie:cookie];
        }
    }
}

- (NSString *)description {
#if TARGET_IPHONE_SIMULATOR
    return [NSString stringWithFormat:@"<%@:%p xingUserID:%@ consumerKey:%@ consumerSecret:%@ accessToken:%@ accessTokenSecret:%@>",
            NSStringFromClass([self class]),
            self,
            self.xingUserID,
            self.consumerKey,
            self.consumerSecret,
            self.accessToken,
            self.accessTokenSecret];
#else
    return [NSString stringWithFormat:@"<%@:%p xingUserID:%@>", NSStringFromClass([self class]), self, self.xingUserID];
#endif
}

- (NSURL *)requestTokenURL {
    return [NSURL URLWithString:[NSString stringWithFormat:@"%@/request_token", self.baseURL]];
}

- (void)viewController:(GTMOAuthViewControllerTouch *)viewController
      finishedWithAuth:(XingOAuthAuthentication *)inAuth
                 error:(NSError *)error {
    self.block(error == nil, error);
}

@end
